{$F+}
unit Main;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, mtbase, ExtCtrls, Buttons, About, Grids;

type
  TThreadDisplay = class(TForm)
    GroupBox1: TGroupBox;
    GroupBox2: TGroupBox;
    Label2: TLabel;
    vStackTop: TLabel;
    Label7: TLabel;
    vStackBottom: TLabel;
    Label1: TLabel;
    vThreadCount: TLabel;
    Label3: TLabel;
    vLastSerialNumber: TLabel;
    GroupBox3: TGroupBox;
    Label4: TLabel;
    vApplications: TLabel;
    SpeedButton1: TSpeedButton;
    Report: TStringGrid;
    procedure SpeedButton1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    Procedure AppIdle(Sender: TObject; var Done: Boolean);
    procedure ReportDblClick(Sender: TObject);
    procedure ReportSelectCell(Sender: TObject; Col, Row: Longint;
      var CanSelect: Boolean);
    procedure FormActivate(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  ThreadDisplay: TThreadDisplay;

implementation

{$R *.DFM}

type
  TThreadMonitor=class(TThread)
  public
    Procedure Run; override;
  end;

const
  ThreadMonitor : TThread=nil;
var
  MCol, MRow : Longint;
  row : integer;

Procedure AddAppInfo(atm : TThreadManager);
var
  appname : array[0..255] of char;
  ctid : longint;
  ct : TThread;
  cts : string;
begin
  GetModuleFileName(atm.handle,appname,sizeof(appname));
  ThreadDisplay.Report.Cells[0,row]:=StrPas(appname);
  ThreadDisplay.Report.Cells[1,row]:=IntToStr(atm.handle);
  ThreadDisplay.Report.Cells[2,row]:='';
  ThreadDisplay.Report.Cells[3,row]:='';
  ThreadDisplay.Report.Cells[4,row]:='';
  ThreadDisplay.Report.Cells[5,row]:='';
  ThreadDisplay.Report.Cells[6,row]:='';
  inc(row);
  { now get thread info and add it }
  for ctid:=0 to atm.ThreadCount-1 do
    begin
      ct:=atm.GetThreadByCount(ctid);
      ThreadDisplay.Report.Cells[0,Row]:=ct.ClassName;
      ThreadDisplay.Report.Cells[1,Row]:=IntToStr(ct.Id);
      if ct.AutoDestroy then
        ThreadDisplay.Report.Cells[2,Row]:='A'
      else
        ThreadDisplay.Report.Cells[2,Row]:=' ';
      If ofs(ct.StackTop^)=0 then
        ThreadDisplay.Report.Cells[3,Row]:='Private'
      else
        ThreadDisplay.Report.Cells[3,Row]:=IntToStr(ofs(ct.StackTop^));
      ThreadDisplay.Report.Cells[4,Row]:=IntToStr(ofs(ct.StackBottom^));
      ThreadDisplay.Report.Cells[5,Row]:=IntToStr(ct.MinSlice);
      ThreadDisplay.Report.Cells[6,Row]:=IntToStr(ct.LastSlice);
      inc(row);
    end;
end;

Procedure TThreadMonitor.Run;
var
  nThreadCount, nLastId, nTicks : longint;
  apid : longint;
  atm : TThreadManager;

begin
  try
    repeat
      nThreadCount:=ThreadCount;
      nLastId:=LastId;
      nTicks:=GetTickCount;
      if CurrentId<>0 then sleep(500);

      { updatedisplay on this count }
      ThreadDisplay.vThreadCount.Caption:=IntToStr(nThreadCount);
      ThreadDisplay.vLastSerialNumber.Caption:=IntToStr(nLastId);
      ThreadDisplay.vApplications.Caption:=IntToStr(ApplicationCount);
      ThreadDisplay.vStackTop.Caption:=IntToStr(MTBase.StackTop);
      ThreadDisplay.vStackBottom.Caption:=IntToStr(MTBase.StackBottom);
      { now blank listbox }
      Row:=ThreadDisplay.Report.FixedRows;
      { Now for each application, starting with ourselves...}
      AddAppInfo(ThreadManager);
      If MTCore then
        begin
          for apid:=0 to ApplicationCount-1 do
            begin
              atm:=GetThreadsByCount(apid);
              If atm.handle<>ThreadManager.Handle then
                AddAppInfo(atm);
            end;
        end;
    ThreadDisplay.Report.RowCount:=Row+ThreadDisplay.Report.FixedRows-1;
    until CurrentId=0;
  except
    MessageBeep(MB_ICONEXCLAMATION);
    Raise;
  end;
end;

procedure TThreadDisplay.SpeedButton1Click(Sender: TObject);
begin
  AboutBox.ShowModal;
end;

procedure TThreadDisplay.FormCreate(Sender: TObject);
begin
  Report.ColWidths[0]:=317;
  Report.ColWidths[1]:=38;
  Report.ColWidths[2]:=18;
  Report.ColWidths[3]:=38;
  Report.ColWidths[4]:=38;
  Report.ColWidths[5]:=38;
  Report.ColWidths[6]:=38;
  Report.Cells[0,0]:='Thread Class Name';
  Report.Cells[1,0]:='ID';
  Report.Cells[2,0]:='AD';
  Report.Cells[3,0]:='Stack';
  Report.Cells[4,0]:='Stack';
  Report.Cells[5,0]:='MSlice';
  Report.Cells[6,0]:='LSlice';
  Application.OnIdle:=AppIdle;
end;

procedure TThreadDisplay.FormDestroy(Sender: TObject);
begin
  If ThreadMonitor=nil then
    exit;
  ThreadMonitor.AutoDestroy:=True;
  RaiseOnThreadId(ThreadMonitor.Id,EAbort.Create('Abort Monitor, Form Destroyed'));
end;

Procedure TThreadDisplay.AppIdle;
begin
  SwitchThreadNow;
  Done:=false;
end;

procedure TThreadDisplay.ReportDblClick(Sender: TObject);
var
  c : longint;
  Tapp : THandle;
  TMan : TThreadManager;
begin
  If MRow>Report.RowCount then exit;
  { now find which application }
  c:=MRow;
  while (copy(Report.Cells[0,c],2,1)<>':') and (c>1) do
    begin
      dec(c);
    end;
  { c is the row with the exe name }
  TApp:=StrToInt(Report.Cells[1,c]);
  TMan:=GetThreadsByHandle(TApp);
  If TMan<>nil then
    begin
      { If they click on the EXE line, kill thread 0, the EXE }
      If C<>MRow then
        c:=StrToInt(Report.Cells[1,MRow])
      else
        c:=0;
      MessageBeep(MB_ICONASTERISK);
      TMan.KillThread(c,5);
    end
  else
    MessageBeep(MB_ICONEXCLAMATION);
end;

procedure TThreadDisplay.ReportSelectCell(Sender: TObject; Col,
  Row: Longint; var CanSelect: Boolean);
begin
  MCol:=Col;
  MRow:=row;
  CanSelect:=true;
end;

procedure TThreadDisplay.FormActivate(Sender: TObject);
begin
  If ThreadMonitor<>nil then
    exit;
  ThreadMonitor:=TThreadMonitor.Create(10,tsApplicationAuto,6000);
  NewThread(ThreadMonitor);
end;

begin
  SetRootStackTop(StackBottom-8192);
end.
